__author__ = 'Calvin Lobo'
__version__ = '1.65'

from .property import Property
from .dictproperty import DictProperty
from .listproperty import ListProperty
from .unitproperty import UnitProperty
from .setproperty import SetProperty
from .limitproperty import LimitProperty
from .stringproperty import StringProperty, _

class BindError(Exception):
    pass


class EventDispatcher(object):
    def __init__(self, **kwargs):
        self.event_dispatcher_event_callbacks = {}
        bindings = {}
        # Walk backwards through the MRO looking for Property attributes in the classes. Then register and bind them to
        # 'on_<prop_name>' if it exists. Walking backwards allows you to override the default value for a superclass.
        for cls in reversed(self.__class__.__mro__):
            for prop_name, prop in cls.__dict__.iteritems():
                if isinstance(prop, Property):
                    prop.name = prop_name
                    prop.register(self, prop_name, prop.default_value)
                    if hasattr(self, 'on_{}'.format(prop_name)):
                        func = getattr(self, 'on_{}'.format(prop_name))
                        bindings.update({prop_name: func})

        self.bind(**bindings)

    def dispatch(self, key, *args):
        for callback in self.event_dispatcher_properties[key]['callbacks']:
            if callback(*args):
                break

    def dispatch_event(self, event, *args):
        for callback in self.event_dispatcher_event_callbacks[event]:
            if callback(*args):
                break

    def register_event(self, name):
        if hasattr(self, 'on_{}'.format(name)):
            self.event_dispatcher_event_callbacks[name] = [getattr(self, 'on_{}'.format(name))]
        else:
            self.event_dispatcher_event_callbacks[name] = []

    def unbind(self, **kwargs):
        all_properties = self.event_dispatcher_properties
        for prop_name, callback in kwargs.iteritems():
            if prop_name in all_properties:
                try:
                    all_properties[prop_name]['callbacks'].remove(callback)
                except ValueError:
                    raise BindError("No binding for {} in property '{}'".format(callback.__name__, prop_name))
            elif prop_name in self.event_dispatcher_event_callbacks:
                try:
                    self.event_dispatcher_event_callbacks[prop_name].remove(callback)
                except ValueError:
                    raise BindError("No binding for {} in event '{}'".format(callback.__name__, prop_name))
            else:
                raise BindError('No property or event by the name of %s' % prop_name)

    def unbind_all(self, *args):
        all_properties = self.event_dispatcher_properties
        for prop_name in args:
            if prop_name in all_properties:
                del all_properties[prop_name]['callbacks'][:]
            elif prop_name in self.event_dispatcher_event_callbacks:
                del self.event_dispatcher_event_callbacks[prop_name][:]
            else:
                raise BindError("No such property or event '%s'" % prop_name)

    def bind(self, **kwargs):
        for prop_name, callback in kwargs.iteritems():
            if prop_name in self.event_dispatcher_properties:
                # Queue the callback into the property
                self.event_dispatcher_properties[prop_name]['callbacks'].append(callback)
            elif prop_name in self.event_dispatcher_event_callbacks:
                # If a property was not found, search in events
                self.event_dispatcher_event_callbacks[prop_name].append(callback)
            else:
                raise BindError("No property or event by the name of '%s'" % prop_name)

    def setter(self, prop_name):
        return lambda inst, value: setattr(self, prop_name, value)

    def get_dispatcher_property(self, prop_name):
        return self.event_dispatcher_properties[prop_name]['property']




__author__ = 'calvin'

from . import EventDispatcher
from .dictproperty import DictProperty


class Selector(EventDispatcher):
    current = DictProperty({})

    def __init__(self, options=[], keys=[], wrap=True, **kwargs):
        super(Selector, self).__init__(**kwargs)
        self.wrap = wrap
        self.keys = []
        self.options = []
        self.register_event('value')
        self.register_event('index')
        self.register_event('key')
        self.set_options(options, keys)

    def set_options(self, options, keys=[]):
        self.options = options
        if keys:
            self.keys = keys
        else:
            self.keys = [str(opt) for opt in options]
        if options:
            self.current.update({'index': 0, 'value': self.options[0], 'key': self.keys[0]})
            # self.dispatch_event('index', self, self.index)

    @property
    def index(self):
        return self.current['index']

    @index.setter
    def index(self, i):
        self.current.update({'index': i, 'value': self.options[i], 'key': self.keys[i]})

    @property
    def value(self):
        return self.current['value']

    @value.setter
    def value(self, x):
        i = self.options.index(x)
        self.current.update({'index': i, 'value': x, 'key': self.keys[i]})

    @property
    def key(self):
        return self.current['key']

    @key.setter
    def key(self, x):
        i = self.keys.index(x)
        self.current.update({'index': i, 'value': self.options[i], 'key': x})

    def on_current(self, inst, current):
        self.dispatch_event('value', inst, current['value'])
        self.dispatch_event('index', inst, current['index'])
        self.dispatch_event('key', inst, current['key'])

    def next(self, *args):
        index = self.index + 1
        if index < len(self.options):
            self.index = index
        elif self.wrap:
            self.index = 0

    def prev(self, *args):
        index = self.index - 1
        if index >= 0:
            self.index = index
        elif self.wrap:
            self.index = len(self.options) - 1

    @property
    def json_repr(self):
        options = [obj.json_repr if hasattr(obj, 'json_repr') else obj for obj in self.options]
        return {'options': options, 'keys': self.keys, 'index': self.index}